package parsekit

// Parser is the top-level struct that holds the configuration for a parser.
// The Parser can be instantiated using the parsekit.NewParser() method.
type Parser struct {
	startHandler ParseHandler // the function that handles the very first state
}

// NewParser instantiates a new Parser.
//
// The Parser is a state machine-style recursive descent parser, in which
// ParseHandler functions are used to move the state machine forward during
// parsing. This style of parser is typically used for parsing programming
// languages and structured data formats (like json, xml, toml, etc.)
//
// To parse input data, use the method Parser.Execute().
func NewParser(startHandler ParseHandler) *Parser {
	return &Parser{startHandler: startHandler}
}

// Execute starts the parser for the provided input.
// When an error occurs during parsing, then this error is returned. Nil otherwise.
func (p *Parser) Execute(input string) *Error {
	api := &ParseAPI{
		input:        input,
		len:          len(input),
		cursorLine:   1,
		cursorColumn: 1,
		loopCheck:    map[string]bool{},
	}
	api.Handle(p.startHandler)
	if !api.stopped && api.err == nil {
		api.UnexpectedInput()
	}
	return api.err
}

// Matcher is the top-level struct that holds the configuration for
// a parser that is based solely on a TokenHandler function.
// The Matcher can be instantiated using the parsekit.NewMatcher()
// method.
//
// To match input data against the wrapped Matcher function, use the method
// Matcher.Parse().
type Matcher struct {
	parser *Parser
	match  string
}

// NewMatcher instantiates a new Matcher.
//
// This is a simple wrapper around a TokenHandler function. It can be used to
// match an input string against that TokenHandler function and retrieve the
// results in a straight forward way.
//
// The 'expects' parameter is used for creating an error message in case parsed
// input does not match the TokenHandler.
func NewMatcher(tokenHandler TokenHandler, expects string) *Matcher {
	matcher := &Matcher{}
	matcher.parser = NewParser(func(p *ParseAPI) {
		if p.On(tokenHandler).Accept() {
			matcher.match = p.BufLiteral()
			p.Stop()
		} else {
			p.Expects(expects)
			p.UnexpectedInput()
		}
	})
	return matcher
}

// Execute feeds the input to the wrapped TokenHandler function.
// It returns the matched input string and an error. When an error
// occurred during parsing, the error will be set, nil otherwise.
func (m *Matcher) Execute(input string) (string, *Error) {
	err := m.parser.Execute(input)
	return m.match, err
}
